## Copyright (C) 2012 Michael Sungjun Kim, mikeskim@gmail.com
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.

## Calculate 'outlier' scores of the sample. Scores are bounded between [0,1].
## Higher scores imply a higher average Kolmogorov-Smirnov test statistic.
## The score is the average sup of the absolute difference of ECDFs of 
## Euclidean distances.  
##
## Description:
## 
##      This function calculates the estimated mean sup of the absolute 
## difference of ECDFs of Euclidean distances via a resampling scheme.   
## 
## Usage:
## 
##      scores = ksetest(x)
## 
## Arguments:
## 
##        x: a vector or matrix of data. For a matrix with n rows and m cols,
##           represents n points living in R^m.             
## 
## Value:
## 
##     scores: a vector of each of the m point's estimated mean sup of the
## absolute difference of ECDFs of Euclidean distances via a resampling scheme.  
## 
## Author(s):
## 
##      Michael Sungjun Kim, mikeskim@gmail.com
##	Original algorithm 12/16/2012
##      White paper forthcoming
## 
## References:
## 
##      Kolmogorov, Smirnov (date unknown)
##      Efron (1979)
## 

function scores = ksetest(x)
warning('off','all');
[nrows, dcols] = size(x);
 if (nrows == 1)
  x = transpose(x);
  nrows = dcols;
  dcols = 1;
 endif 

scores = zeros(nrows,1);
nsample = ceil( nrows*0.95);
 if (nsample > 300)
  nsample = 300;
 endif

for i = 1:nrows
 dpop = x(randperm(nrows)(1:nsample),:); %##sample points
 distSample0 = zeros(nsample,1);
 
 for j = 1:nsample %##build distances from point i to sampled points
  distSample0(j,1) = sqrt(sum((dpop(j,:) - x(i,:)).^2)); 
 endfor

 bpop = x(randperm(nrows)(1:nsample),:);  
 for k = 1:nsample  %##build distances from bpop k to sampled points dpop
  distSampleTemp = zeros(nsample,1); 
   for j = 1:nsample
    distSampleTemp(j,1) = sqrt(sum(( dpop(j,:) -  bpop(k,:)).^2));
   endfor
  [pval, ks, d] = kolmogorov_smirnov_test_2(distSample0,distSampleTemp);
  scores(i,1) = scores(i,1) + d/nsample;
 endfor
 
endfor
warning('on','all');
endfunction
